package com.guildasoft.bluechatroom2.utils;

import com.guildasoft.bluechatroom2.entity.AppMessage;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Optional;

public class MessageUtilsImpl implements MessageUtils {
    private static final String MESSAGE_OK_IDENTIFIER = "⬚";
    private static final String MESSAGE_SPLITTING_CHAR = "⍭↭⍭";
    public static final int MAX_TEXT_SIZE = 900;
    private static final int MAX_MESSAGE_SIZE = 984;
    private final int TYPE_INDEX = 0;
    private final int UUID_INDEX = 1;
    private final int USER_INDEX = 2;
    private final int TEXT_INDEX = 3;
    private final int MAX_INDEX = 4;

    public enum MessageTypes {
        CHAT_ORIGIN(0), CHAT_SHARE(1),
        INFO_ARRIVAL(2), INFO_FAREWELL(3),
        ERROR(4), NONE(5);

        private final int value;

        MessageTypes(int i) {
            this.value = i;
        }

        public int getValue() {
            return value;
        }

        public static Optional<MessageTypes> valueOf(int value){
            return Arrays.stream(values()).filter(type -> type.value == value).findFirst();
        }
    }

    @Override
    public String getUuidFromMessage(String message) {
        String[] parts = message.split(MESSAGE_SPLITTING_CHAR, MAX_INDEX);
        return parts[UUID_INDEX];
    }

    @Override
    public String getTextFromMessage(String message) {
        String[] parts = message.split(MESSAGE_SPLITTING_CHAR, MAX_INDEX);
        return parts[TEXT_INDEX];
    }

    @Override
    public String getUserFromMessage(String message) {
        String[] parts = message.split(MESSAGE_SPLITTING_CHAR, MAX_INDEX);
        return parts[USER_INDEX];
    }

    @Override
    public MessageTypes getTypeFromMessage(String message) {
        String[] parts = message.split(MESSAGE_SPLITTING_CHAR, MAX_INDEX);
        int valor = Integer.parseInt(parts[TYPE_INDEX].substring(1));
        return MessageTypes.valueOf(valor).orElse(MessageTypes.NONE);
    }

    public Boolean messageContainsOkIdentifier(String message){
        return message.startsWith(MESSAGE_OK_IDENTIFIER);
    }

    @Override
    public String mountCompleteMessage(MessageTypes type, String uuid, String name, String text) {
        return String.format(
                "%s%s%s%s%s%s%s%s", MESSAGE_OK_IDENTIFIER, type.getValue(),
                                    MESSAGE_SPLITTING_CHAR, uuid,
                                    MESSAGE_SPLITTING_CHAR, name,
                                    MESSAGE_SPLITTING_CHAR, text
        );
    }

    @Override
    public String mountCompleteMessage(MessageTypes type, String uuid, String name) {
        return String.format(
                "%s%s%s%s%s%s%s%s", MESSAGE_OK_IDENTIFIER, type.toString(),
                                    MESSAGE_SPLITTING_CHAR, uuid,
                                    MESSAGE_SPLITTING_CHAR, name,
                                    MESSAGE_SPLITTING_CHAR, ""
        );
    }

    @Override
    public AppMessage convertMessageBytesToObject(byte[] bytes, int size) {
        String receivedMessage = new String(bytes, 0, size, StandardCharsets.UTF_8);
        String miliseconds = String.valueOf(System.currentTimeMillis());
        if(receivedMessage.startsWith(MESSAGE_OK_IDENTIFIER)) {
            String text = getTextFromMessage(receivedMessage).length() > MAX_TEXT_SIZE ? getTextFromMessage(getTextFromMessage(receivedMessage).substring(0, MAX_MESSAGE_SIZE)) : getTextFromMessage(receivedMessage);
            return AppMessage.builder()
                    .messageType(getTypeFromMessage(receivedMessage))
                    .owner(getUserFromMessage(receivedMessage))
                    .text(text)
                    .uuid(getUuidFromMessage(receivedMessage))
                    .time(miliseconds)
                    .build();
        }
        return AppMessage.builder().messageType(MessageTypes.NONE).build();
    }

    @Override
    public String convertAppMessageToString(AppMessage message) {
        return mountCompleteMessage(
                message.messageType, message.uuid, message.owner, message.text
        );
    }

    @Override
    public String mountMessageUsingAppMessage(AppMessage appMessage) {
        return mountCompleteMessage(appMessage.messageType, appMessage.uuid, appMessage.owner, appMessage.text);
    }

}
